@@ -1,5 +1,9 @@
 import copy
-from omgeo.processors.postprocessors import DupePicker
+import logging
+from omgeo.processors.postprocessors import DupePicker, SnapPoints
+import time
+
+logger = logging.logger = logging.getLogger(__name__)
 
 class Geocoder():
     """
@@ -11,21 +15,25 @@ class Geocoder():
     sources         -- a dictionary of GeocodeServiceConfig() parameters,
                        keyed by module name for the GeocodeService to use
                        ex: {'esri_na':{}, 
-                            'bing': {
-                             'settings': {},
-                             'preprocessors': [],
-                             'postprocessors': []}, ...}
+                            'bing': {'settings': {},
+                                     'preprocessors': [],
+                                     'postprocessors': []},
+                            ...}
     preprocessors   -- list of universal preprocessors to use
     postprocessors  -- list of universal postprocessors to use
     """
 
     DEFAULT_SOURCES = [['omgeo.services.EsriNA', {}],
-                        ['omgeo.services.EsriEU', {}],
-                        ['omgeo.services.Nominatim', {}]]
+                       ['omgeo.services.EsriEU', {}],
+                       ['omgeo.services.Nominatim', {}]]
     DEFAULT_PREPROCESSORS = []
-    DEFAULT_POSTPROCESSORS = [
-        DupePicker('match_addr', 'locator', ['rooftop', 'parcel', 'interpolation_offset', 'interpolation']),
-    ]
+    DEFAULT_POSTPROCESSORS = [SnapPoints(),
+                              DupePicker('match_addr', 'locator',
+                                         ['rooftop',
+                                          'parcel',
+                                          'interpolation_offset',
+                                          'interpolation'])]
+    
     def _get_service_by_name(self, service_name):
         try:
             module, separator, class_name = service_name.rpartition('.')
@@ -60,7 +68,6 @@ class Geocoder():
         for source in sources: # iterate through a list of sources
             self.add_source(source)
 
-
     def __init__(self, sources=None, preprocessors=None, postprocessors=None):
         self._preprocessors = Geocoder.DEFAULT_PREPROCESSORS \
             if preprocessors is None else preprocessors
@@ -86,6 +93,7 @@ class Geocoder():
                         the first geocoding service with valid candidates
                         (default False).
         """
+        start_time = time.time()
         waterfall = self._settings.get('waterfall', False)
         processed_pq = copy.copy(pq)
         for p in self._preprocessors: # apply universal address preprocessing
@@ -94,12 +102,13 @@ class Geocoder():
 
         processed_candidates = []
         for gs in self._sources: # iterate through each GeocodeService
+            logger.debug('%s: Geocoding using %s...' % ((time.time() - start_time), gs))
             candidates = gs.geocode(processed_pq)
             processed_candidates += candidates # merge lists
             if waterfall is False and len(processed_candidates) > 0:
-                break # if we have >= 1 good candidate, don't go to next geocoder
+                break # if >= 1 good candidate, don't go to next geocoder
 
-        for p in self._postprocessors: # apply universal candidate postprocessing
+        for p in self._postprocessors: # apply univ. candidate postprocessing
+            logger.debug('%s: Applying universal postprocessor %s...' % ((time.time() - start_time), p))
             processed_candidates = p.process(processed_candidates) 
-
         return processed_candidates
