@@ -1,11 +1,10 @@
 from base import GeocodeService
+import logging
 from omgeo.places import Candidate
 from omgeo.processors.preprocessors import CountryPreProcessor, RequireCountry, ParseSingleLine, ReplaceRangeWithNumber
 from omgeo.processors.postprocessors import AttrFilter, AttrExclude, AttrRename, AttrSorter, AttrMigrator, UseHighScoreIfAtLeast, GroupBy, ScoreSorter
-
 from suds.client import Client
-
-import logging
+import time
 
 logger = logging.getLogger(__name__)
 
@@ -13,7 +12,7 @@ class Bing(GeocodeService):
     """
     Class to geocode using Bing services:
      * `Find a Location by Query <http://msdn.microsoft.com/en-us/library/ff701711.aspx>`_
-     . * `Find a Location by Address <http://msdn.microsoft.com/en-us/library/ff701714.aspx>`_
+     * `Find a Location by Address <http://msdn.microsoft.com/en-us/library/ff701714.aspx>`_
 
     Settings used by the Bing GeocodeService object may include:
     ============================================================
@@ -21,63 +20,74 @@ class Bing(GeocodeService):
     """
 
     _endpoint = 'http://dev.virtualearth.net/REST/v1/Locations'
-    
-    LOCATOR_MAP = {
-        'Rooftop': 'rooftop',
-        'Parcel': 'parcel',
-        'ParcelCentroid': 'parcel',
-        'Interpolation': 'interpolation',
-        'InterpolationOffset': 'interpolation_offset',
-    }
-
-    DEFAULT_REJECTS = ['AdminDivision1', 'AdminDivision2', 'AdminDivision3', 'CountryRegion', 'DisputedArea', 'MountainRange', 'Ocean', 'Peninsula', 'Planet', 'Plate', 'Postcode', 'Postcode1', 'Postcode2', 'Postcode3', 'Postcode4', 'Sea']
 
     DEFAULT_PREPROCESSORS = [
         ReplaceRangeWithNumber()
     ]
     
     DEFAULT_POSTPROCESSORS = [
-        AttrExclude(DEFAULT_REJECTS, 'entity'),
-        AttrRename('locator', LOCATOR_MAP),
-        AttrSorter(['rooftop', 'parcel', 'interpolation_offset', 'interpolation'], 'locator'),
-        AttrSorter(['Address'], 'entity'),
-        AttrMigrator('confidence', 'score', {'High':100, 'Medium':85, 'Low':50}),
-        ScoreSorter(),
-        GroupBy('match_addr')
-    ]
+       AttrMigrator('confidence', 'score',
+                    {'High':100, 'Medium':85, 'Low':50}),
+       UseHighScoreIfAtLeast(100),
+       AttrFilter(['Address', 'AdministrativeBuilding', 
+                   'AgriculturalStructure',
+                   'BusinessName', 'BusinessStructure',
+                   'BusStation', 'Camp', 'Church', 'CityHall',
+                   'CommunityCenter', 'ConventionCenter',
+                   'Courthouse', 'Factory', 'FerryTerminal',
+                   'FishHatchery', 'Fort', 'Garden', 'Geyser',
+                   'Heliport', 'IndustrialStructure',
+                   'InformationCenter', 'Junction',
+                   'LandmarkBuilding', 'Library', 'Lighthouse',
+                   'Marina', 'MedicalStructure', 'MetroStation',
+                   'Mine', 'Mission', 'Monument', 'Mosque', 
+                   'Museum', 'NauticalStructure', 'NavigationalStructure',
+                   'OfficeBuilding', 'ParkAndRide', 'PlayingField',
+                   'PoliceStation', 'PostOffice', 'PowerStation',
+                   'Prison', 'RaceTrack', 'ReligiousStructure',
+                   'RestArea', 'Ruin', 'ShoppingCenter', 'Site',
+                   'SkiArea', 'Spring', 'Stadium', 'Temple',
+                   'TouristStructure'], 'entity'),
+       AttrRename('locator', dict(Rooftop='rooftop',
+                                  Parcel='parcel',
+                                  ParcelCentroid='parcel',
+                                  Interpolation='interpolation',
+                                  InterpolationOffset='interpolation_offset')),
+       AttrSorter(['rooftop', 'parcel', 
+                   'interpolation_offset', 'interpolation'],
+                   'locator'),
+       AttrSorter(['Address'], 'entity'),
+       ScoreSorter(),
+       GroupBy(('x', 'y')),
+       GroupBy('match_addr')]
+    DEFAULT_POSTPROCESSORS = []
     
     def __init__(self, preprocessors=None, postprocessors=None, settings=None):
-
         preprocessors = Bing.DEFAULT_PREPROCESSORS if preprocessors is None else preprocessors
         postprocessors = Bing.DEFAULT_POSTPROCESSORS if postprocessors is None else postprocessors
-
         GeocodeService.__init__(self, preprocessors, postprocessors, settings)
 
     def _geocode(self, pq):
         if pq.query.strip() == '':
             # No single line query string; use address elements:
-            query = {
-                'addressLine':pq.address,
-                'locality':pq.city,
-                'adminDistrict':pq.state,
-                'postalCode':pq.postal,
-                'countryRegion':pq.country}
+            query = {'addressLine':pq.address,
+                     'locality':pq.city,
+                     'adminDistrict':pq.state,
+                     'postalCode':pq.postal,
+                     'countryRegion':pq.country}
         else:
-            query = {
-                'query':pq.query}
+            query = {'query':pq.query}
         
         if pq.viewbox is not None:
             query = dict(query, **{'umv':pq.viewbox.to_bing_str()})
-
         if hasattr(pq, 'culture'): query = dict(query, c=pq.culture)
         if hasattr(pq, 'user_ip'): query = dict(query, uip=pq.user_ip)
         if hasattr(pq, 'user_lat') and hasattr(pq, 'user_lon'):
             query = dict(query, **{'ul':'%f,%f' % (pq.user_lat, pq.user_lon)})
 
-        addl_settings = {
-                'key':self._settings['api_key']}
+        addl_settings = {'key':self._settings['api_key']}
         query = dict(query, **addl_settings)
-        
+        logger.debug(query)
         response_obj = self._get_json_obj(self._endpoint, query)
         if response_obj is False: return []
   
@@ -113,11 +123,11 @@ class EsriGeocodeService(GeocodeService):
         service up with the premium task URL.
         """
         GeocodeService.__init__(self, preprocessors, postprocessors, settings)
-
-        service_url = 'http://tasks.arcgisonline.com/ArcGIS'
         
         if 'api_key' in self._settings:
             service_url = 'http://premiumtasks.arcgisonline.com/server'
+        else:
+            service_url = 'http://tasks.arcgisonline.com/ArcGIS'
 
         self._endpoint = service_url + self._task_endpoint
 
@@ -162,7 +172,6 @@ class EsriSoapGeocodeService(EsriGeocodeService):
             ps.Value = v
             props.append(ps)
         return props
-
     
     def _get_candidates_from_record_set(self, record_set):
         """
@@ -196,15 +205,15 @@ class EsriEUGeocodeService():
     """
     _wkid = 4326
 
-    _SUPPORTED_COUNTRIES_FIPS = ['AN', 'AU', 'BE', 'DA', 'FI', 'FR', 'GM', 'GI', 'EI', 
+    SUPPORTED_COUNTRIES_FIPS = ['AN', 'AU', 'BE', 'DA', 'FI', 'FR', 'GM', 'GI', 'EI', 
         'IT', 'LS', 'LU', 'MN', 'NL', 'NO', 'PO', 'SM', 'SP', 'SW', 'SZ', 'UK', 'VT']
     """FIPS codes of supported countries"""
 
-    _SUPPORTED_COUNTRIES_ISO2 = ['AD', 'AT', 'BE', 'DK', 'FI', 'FR', 'DE', 'GI', 'IE', 
+    SUPPORTED_COUNTRIES_ISO2 = ['AD', 'AT', 'BE', 'DK', 'FI', 'FR', 'DE', 'GI', 'IE', 
         'IT', 'LI', 'LU', 'MC', 'NL', 'NO', 'PT', 'SM', 'ES', 'SE', 'CH', 'GB', 'VC']
     """ISO-2 codes of supported countries"""
 
-    _MAP_FIPS_TO_ISO2 = {
+    MAP_FIPS_TO_ISO2 = {
         'AN':'AD',
         'AU':'AT',
         'DA':'DK',
@@ -227,8 +236,8 @@ class EsriEUGeocodeService():
 
     DEFAULT_PREPROCESSORS = [
         CountryPreProcessor(
-            _SUPPORTED_COUNTRIES_ISO2,
-            _MAP_FIPS_TO_ISO2),
+            SUPPORTED_COUNTRIES_ISO2,
+            MAP_FIPS_TO_ISO2),
         ParseSingleLine(),
         RequireCountry('GB'),
     ]
@@ -381,7 +390,7 @@ class EsriNASoap(EsriSoapGeocodeService, EsriNAGeocodeService):
         if location.query:
             # Single line geocoding
             location_dict = {
-                'Single Line Input': location.query
+                'SingleLine': location.query
             }
         else:
             # Split address
@@ -392,11 +401,6 @@ class EsriNASoap(EsriSoapGeocodeService, EsriNAGeocodeService):
                 'Zip': location.postal
             }
 
-            # Check for full postal codes
-            if location.country.lower == 'us' and len(location.postal > 9):
-                location_dict['Zip'] = location.postal[:5]
-                location_dict['Zip4'] = location.postal[-4:]
-
         address.PropertyArray.PropertySetProperty.append(
                 self._get_property_set_properties(location_dict))
 
@@ -412,6 +416,7 @@ class EsriNASoap(EsriSoapGeocodeService, EsriNAGeocodeService):
         
         
 class EsriNA(EsriGeocodeService, EsriNAGeocodeService):
+    """Esri REST Geocoder for North America"""
     _task_endpoint = '/rest/services/Locators/TA_Address_NA_10/GeocodeServer/findAddressCandidates'
             
     def __init__(self, preprocessors=None, postprocessors=None, settings=None):
@@ -469,21 +474,27 @@ class Nominatim(GeocodeService):
 
     _endpoint = 'http://open.mapquestapi.com/nominatim/v1/search'
 
-    DEFAULT_ACCEPTED_ENTITIES = ['building.', 'historic.castle', 'leisure.ice_rink', 'leisure.miniature_golf', 'leisure.sports_centre', 'lesiure.stadium', 'leisure.track', 'lesiure.water_park', 'man_made.lighthouse', 'man_made.works', 'military.barracks', 'military.bunker', 'office.', 'place.house', 'amenity.',  'power.generator', 'railway.station', 'shop.', 'tourism.']
+    DEFAULT_ACCEPTED_ENTITIES = ['building.', 'historic.castle', 'leisure.ice_rink', 
+                                 'leisure.miniature_golf',
+                                 'leisure.sports_centre', 'lesiure.stadium', 'leisure.track',
+                                 'lesiure.water_park', 'man_made.lighthouse', 'man_made.works', 
+                                 'military.barracks', 'military.bunker', 'office.', 'place.house',
+                                 'amenity.',  'power.generator', 'railway.station',
+                                 'shop.', 'tourism.']
 
     DEFAULT_REJECTED_ENTITIES = ['amenity.drinking_water',
-'amentity.bicycle_parking', 'amentity.ev_charging', 'amentity.grit_bin',
-'amentity.atm', 'amentity.hunting_stand', 'amentity.post_box']
+                                 'amentity.bicycle_parking', 'amentity.ev_charging',
+                                 'amentity.grit_bin', 'amentity.atm',
+                                 'amentity.hunting_stand', 'amentity.post_box']
 
+    DEFAULT_PREPROCESSORS = [ReplaceRangeWithNumber()] # 766-68 Any St. -> 766 Any St. 
     """Preprocessors to use with this geocoder service, in order of desired execution."""
-    DEFAULT_PREPROCESSORS=ReplaceRangeWithNumber() # 766-68 Any St. -> 766 Any St. 
     
-    """Postprocessors to use with this geocoder service, in order of desired execution."""
-
     DEFAULT_POSTPROCESSORS = [
         AttrFilter(DEFAULT_ACCEPTED_ENTITIES, 'entity', exact_match=False),
         AttrExclude(DEFAULT_REJECTED_ENTITIES, 'entity')
     ]
+    """Postprocessors to use with this geocoder service, in order of desired execution."""
     
     def __init__(self, preprocessors=None, postprocessors=None, settings=None):
         preprocessors = Bing.DEFAULT_PREPROCESSORS if preprocessors is None else preprocessors
@@ -491,10 +502,9 @@ class Nominatim(GeocodeService):
         GeocodeService.__init__(self, preprocessors, postprocessors, settings)
 
     def _geocode(self, pq):
-        query = {
-            'q':pq.query,
-            'countrycodes':pq.country, # only takes ISO-2
-            'format':'json'}
+        query = {'q':pq.query,
+                 'countrycodes':pq.country, # only takes ISO-2
+                 'format':'json'}
         
         if pq.viewbox is not None:
             query = dict(query, **{'viewbox':pq.viewbox.to_mapquest_str(), 'bounded':pq.bounded})
